@@ -1,4 +1,3 @@
-/* $cyphertite: ctutil.h,v 1.5 2011/07/19 21:54:23 oga Exp $ */
 /*
  * Copyright (c) 2011 Conformal Systems LLC <info@conformal.com>
  *
@@ -117,6 +116,8 @@ struct ct_nop_reply {
  */
 #define C_HDR_O_LOGIN		(4)
 #define C_HDR_O_LOGIN_REPLY	(5)
+#define C_HDR_O_LOGIN_EXS_DISABLED	(1)
+extern char	*c_hdr_login_reply_ex_errstrs[];
 
 /*
  * C_O_EXISTS: ask server if digest exists
@@ -197,6 +198,9 @@ void		ct_sha1_setup(SHA_CTX *);
 void		ct_sha1_add(uint8_t *, SHA_CTX *, size_t);
 void		ct_sha1_final(uint8_t *, SHA_CTX *);
 
+/* error handling */
+char		*ct_header_strerror(struct ct_header *);
+
 /* protocol */
 void		ct_wire_header(struct ct_header *);
 void		ct_unwire_header(struct ct_header *);
@@ -216,13 +220,44 @@ void		ct_dump_block(uint8_t *p, size_t sz);
 #define CT_B64_INVALID		(0)
 #define CT_B64_ENCODE		(1)
 #define CT_B64_DECODE		(2)
+#define CT_B64_M_ENCODE		(3)
+#define CT_B64_M_DECODE		(4)
 
 int		ct_base64_encode(int, uint8_t *, size_t, uint8_t *, size_t);
 
 /* Metadata */
-#define CT_MAX_MD_FILENAME	(256)
+#define CT_MAX_MD_FILENAME	(256)	/* 255 + terminating NUL */
+#define	CT_MD_TAG_MAXLEN	(129)	/* 128 + terminating NUL */
+#define	CT_MD_TAG_REJECTCHRS	"?!*/\\\'\""
+
+/* filename handling */
+char	*ct_remove_ext(char *path);
 
 /* directory handling */
 int     ct_make_full_path(char *, mode_t);
 
+/* pipe handling */
+int	 ct_set_pipe_nonblock(int);
+
+/* cli parsing */
+struct ct_cli_cmd {
+	char			*cc_cmd;	/* command name */
+	struct ct_cli_cmd	*cc_subcmd;	/* subcommand structure */
+	int			cc_paramc;	/* number of parameters */
+#define CLI_CMD_SUBCOMMAND	(-2)	/* dereference cmd in substructure */
+#define CLI_CMD_UNKNOWN		(-1)    /* unknown number of parameters */
+	char			*cc_usage;	/* command usage string */
+	void			(*cc_cb)(struct ct_cli_cmd *, int, char **);
+	int			cc_auth;	/* need authentication */
+};
+
+struct ct_cli_cmd	*ct_cli_cmd_find(struct ct_cli_cmd *, char *);
+__dead void		ct_cli_usage(struct ct_cli_cmd *, struct ct_cli_cmd *);
+struct ct_cli_cmd	*ct_cli_validate(struct ct_cli_cmd *, int *, char ***);
+void			ct_cli_execute(struct ct_cli_cmd *, int *, char ***);
+
+/* Debug log levels */
+#define CTUTIL_LOG_SOCKET	(0x1)
+#define CTUTIL_LOG_CONFIG	(0x2)
+
 #endif /* CTUTIL_H */
